package de.elisabetheckstaedt.dipl.common;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.StringReader;
import java.text.DecimalFormat;
import java.util.Arrays;
import java.util.Locale;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import de.elisabetheckstaedt.dipl.real.AbKM;
import de.elisabetheckstaedt.dipl.real.Kaeltemaschine;
import de.elisabetheckstaedt.dipl.real.Teilsystem;
import de.elisabetheckstaedt.dipl.real.Luftrueckkuehler;
import de.elisabetheckstaedt.dipl.real.Subnetz;
import de.elisabetheckstaedt.dipl.real.Verbraucher;
import de.elisabetheckstaedt.dipl.real.Verdunstungsrueckkuehler;
import de.elisabetheckstaedt.dipl.real.Wasserrueckkuehler;

/**
 * Diese Klasse speichert die Systemkonfigurationab. Sie enthlt Methoden zu deren Berechnung und 
 * Ausgabe der Ergebnisse.
 * @author Elisabeth Eckstdt
 */
public class Gesamtsystem extends Sammelobjekt{
	
	/**
	 * Vektor aller Teilsysteme
	 */
	public Teilsystem[] teilsysteme;
	
	public final Projekt projekt;
	
	public Gesamtsystem(Projekt projekt, String Name) {
		super(Name);
		this.projekt = projekt;
	}
	
	public Object[] getStruktur()
	{
		Object[] versammelt = new Object[1];
		versammelt[0] = teilsysteme;
		return versammelt;
	}
	
	public String getBauteilart()
	{
		return "Gesamtsystem";
	}
	
	public void verbrauecheBerechnen(Versorgungsaufgabe versorgungsaufgabe)
	{
		for (int i=0; i<teilsysteme.length; i++)
		{
			teilsysteme[i].verbraeucheBerechnen(versorgungsaufgabe);
		}
	}
	
	/**
	 * Dateianlage funktioniert bis jetzt nur unter Windows
	 * @param projekt
	 */
	public void XMLout(Projekt projekt)
	{
		//Dokument erzeugen
		DocumentBuilderFactory docBFac;
		DocumentBuilder docBuild = null;
		try
		{
			docBFac = DocumentBuilderFactory.newInstance();
			docBuild = docBFac.newDocumentBuilder();
		}
		catch( Exception e )
		{
			e.printStackTrace();
			System.out.println("Dokument konnte nicht erzeugt werden");
		}
		Document baum = docBuild.newDocument();
		
		//Dokument fllen
        Element root = baum.createElement("Loesungsvariante");
        root.setAttribute("Name", Name);
        //Aufgabenstellung
	        Element Aufgabenst = baum.createElement("Aufgabenstellung");
	        	//Klima
	        	Element Klima = baum.createElement("Klimadatensatz");
	        	Klima.setAttribute("Name", projekt.getKDS().getName());
        	Aufgabenst.appendChild(Klima);
        	Element Preisdaten = baum.createElement("Preisdaten");
	        	Preisdaten.setAttribute("Betrachtungszeitraum", Helper.Zahlformat(projekt.getPD().getBetrachtungszeitraum(),0));
	        	Preisdaten.setAttribute("Zinssatz", Helper.Zahlformat(projekt.getPD().getZinssatz(),3));
	        	Preisdaten.setAttribute("Preisaenderungsfaktor_Investitionen", Helper.Zahlformat(projekt.getPD().getPaf_invest(),3));
	        	Helper.Wertattribut(Preisdaten, "Stundenlohn", projekt.getPD().getStundenlohn());
	        	Preisdaten.setAttribute("Preisaenderungsfaktor_Lohn", Helper.Zahlformat(projekt.getPD().getPaf_lohn(),3));
	        	Preisdaten.setAttribute("Strompreis", Helper.Euroformat((projekt.getPD().getStrompreis()[0]),3));
	        	Preisdaten.setAttribute("Waermepreis", Helper.Euroformat((projekt.getPD().getWaermepreis()[0]),3));
	        	Preisdaten.setAttribute("Wasserpreis", Helper.Euroformat((projekt.getPD().getWasserpreis()[0]),3));
	        	Preisdaten.setAttribute("Preisaenderungsfaktor_Energie", Helper.Zahlformat(projekt.getPD().getPaf_energie(),3));
	        Aufgabenst.appendChild(Preisdaten);
	        
        root.appendChild(Aufgabenst);
        
        Element investition = baum.createElement("Investition");
        investition.setAttribute("Investition:", Helper.Euroformat(getInvestition(), 0));
        root.appendChild(investition);
        
        //Lsung        
        root.appendChild(KostenXMLout(baum, projekt.getPD()));
        baum.appendChild(root);

		//in XML-Datei ausgeben
        DOMSource source = new DOMSource(baum);
        File test = new File("detailliert.xml");
			try {
				test.createNewFile();
			} catch (IOException e) {e.printStackTrace();}
        System.out.println(test.getAbsoluteFile());
		StreamResult result = new StreamResult(test);
		Transformer transformer = null;
		try
		{
			transformer = TransformerFactory.newInstance().newTransformer();
		}
		catch (TransformerConfigurationException e1)
		{
			e1.printStackTrace();
		}
		catch (TransformerFactoryConfigurationError e1)
		{
			e1.printStackTrace();
		}
		try
		{
			transformer.transform(source, result);
		}
		catch (TransformerException e1)
		{
			e1.printStackTrace();
		}
	}
	public Node KostenXMLout(Document baum, Preisdaten preisdaten)
	{
		
		Element aktElem = baum.createElement(getBauteilart());
		Helper.Wertattribut(aktElem, "Kosten", getKosten());
		
		Helper.Wertattribut(aktElem, "Kapitalgebundene_Kosten", getKapitalkosten());
		Helper.Wertattribut(aktElem, "Investition", getInvestitionskosten());
    	Helper.Wertattribut(aktElem, "Instandsetzung", getInstandsetzungskosten());
    	
    	Helper.Wertattribut(aktElem, "Betriebsgebundene_Kosten", getBetriebsgebundene_Kosten());
    	Helper.Wertattribut(aktElem, "Bedienung", getBedienungskosten());
    	Helper.Wertattribut(aktElem, "Wartung", getWartungskosten());
    	Helper.Wertattribut(aktElem, "Inspektion", getInspektionskosten());
    	
    	Helper.Wertattribut(aktElem, "Verbrauchsgebundene_Kosten", getVerbrauchsgebundene_Kosten());
    	Helper.Wertattribut(aktElem, "Stromkosten", getStromkosten());
        Helper.Wertattribut(aktElem, "Waermekosten", getWaermekosten());
        Helper.Wertattribut(aktElem, "Wasserkosten", getWasserkosten());
        
		aktElem.setAttribute("Sammelname", getName());
		
		Object[] Bestandteile = getStruktur();
		for (int i=0; i<Bestandteile.length; i++)
		{
			Kostenstelle[] Kostentraegerliste = ((Kostenstelle[])Bestandteile[i]);
			if (Kostentraegerliste.length>1)
			{
				Element Kindelemente = baum.createElement("alle"+Kostentraegerliste[0].getBauteilart());
				for (int j=0; j<Kostentraegerliste.length; j++)
				{
					Kostenstelle kt = ((Kostenstelle[]) Bestandteile[i])[j];
					Node test = kt.KostenXMLout(baum, preisdaten);
					Kindelemente.appendChild(test);
				}
				aktElem.appendChild(Kindelemente);
			}
			else
			{
				aktElem.appendChild(Kostentraegerliste[0].KostenXMLout(baum, preisdaten));
			}
		}
		return aktElem;
	}
	/**
	 * Dateianlage funktioniert bis jetzt nur unter Windows
	 * @param preisdaten
	 */
	public void XMLoutflach(Preisdaten preisdaten)
	{
		//Dokument erzeugen
		DocumentBuilderFactory docBFac;
		DocumentBuilder docBuild = null;
		try
		{
			docBFac = DocumentBuilderFactory.newInstance();
			docBuild = docBFac.newDocumentBuilder();
			
		}
		catch( Exception e )
		{
			e.printStackTrace();
			System.out.println("Dokument konnte nicht erzeugt werden");
		}
		Document baum = docBuild.newDocument();
		
		//Dokument fllen
        Element root = baum.createElement("Loesungsvariante");
        root.setAttribute("Name", Name);
        root = (Element) KostenXMLoutflach(baum, root, "", "", preisdaten);
        baum.appendChild(root);
        
		//in XML-Datei ausgeben
        DOMSource source = new DOMSource(baum);
        File tmpFile = new File( "pivot_optimiert.xml");
        try {
			tmpFile.createNewFile();
		} catch (Exception e) {
			e.printStackTrace();
		}
		StreamResult result = new StreamResult(tmpFile);
		Transformer transformer = null;
		try
		{
			transformer = TransformerFactory.newInstance().newTransformer();
		}
		catch (TransformerConfigurationException e1)
		{
			e1.printStackTrace();
		}
		catch (TransformerFactoryConfigurationError e1)
		{
			e1.printStackTrace();
		}
		try
		{
			transformer.transform(source, result);
		}
		catch (TransformerException e1)
		{
			e1.printStackTrace();
		}
	}

	public void setTeilsysteme(Teilsystem[] teilsysteme) {
		this.teilsysteme = teilsysteme;
	}
	
	/**
	 * Dateianlage funktioniert bis jetzt nur unter Windows
	 */
	public void zeitschritte_out()
	{
		 File ziel = new File("zeitschritte.dat");
    	try {
			ziel.createNewFile();
		} catch (IOException e2) {			e2.printStackTrace();		}
		//Vektoren sammeln
		double[] t_A = projekt.getKDS().getAussentemperatur();
		double[][][][][] lvvektoren = new double[teilsysteme.length][2][][][];
		for (int i=0; i<teilsysteme.length; i++)
		{
			Teilsystem kz = teilsysteme[i];
			lvvektoren[i][0] = new double[kz.getKaeltemaschinen().length][][];
			for (int j=0; j<kz.getKaeltemaschinen().length; j++)
			{
				Kaeltemaschine km = kz.getKaeltemaschinen()[j];
				double[][] kmvektoren;
				double[] kl = kz.Kaelteleistungen(j, t_A);
				double[] Belastung;
				if (km.getRueckkuehler() instanceof Wasserrueckkuehler)
				{
					Wasserrueckkuehler rk = (Wasserrueckkuehler) km.getRueckkuehler();
					kmvektoren = new double[9][];
					Belastung = kl;
					double[] Wasserverbrauch = new double[8760];
					if (rk instanceof Verdunstungsrueckkuehler)
						Wasserverbrauch = ((Verdunstungsrueckkuehler) rk).getWasserverbrauch();
					else
						Arrays.fill(Wasserverbrauch,0);
					kmvektoren[5] = Wasserverbrauch;
					kmvektoren[6] = rk.getStromverbrauch();
					double[] t_W_E = rk.t_W_E(km.Rueckkuehlleistung(kl));
					kmvektoren[7] = rk.f_R(projekt.getKDS(), t_W_E, rk.t_W_A());
					kmvektoren[8] = t_W_E;
				}
				else
				{
					kmvektoren = new double[5][];
					Belastung = t_A;
				}
				kmvektoren[0] = kl;
				kmvektoren[1] = km.PLV(Belastung);
				kmvektoren[2] = km.getStromverbrauch();
				double[] Waermeverbrauch = new double[8760];
				if (km instanceof AbKM)
					Waermeverbrauch = ((AbKM) km).getWaermeverbrauch();
				else
					Arrays.fill(Waermeverbrauch,0);
				kmvektoren[3] = Waermeverbrauch;
				kmvektoren[4] = km.Rueckkuehlleistung(kl);
				lvvektoren[i][0][j] = kmvektoren;
			}
			lvvektoren[i][1] = new double[kz.getSubnetze().length][][];
			for (int j=0; j<kz.getSubnetze().length; j++)
			{
				Subnetz tn = kz.getSubnetze()[j];
				double[][] tnvektoren = new double[3][];
				double[] vp = tn.volumenstrom(t_A, kz.getT_K_A());
				double[] dp = tn.Druckverlust(vp);
				tnvektoren[0] = vp;
				tnvektoren[1] = dp;
				tnvektoren[2] = tn.getPumpen().getStromverbrauch();
				lvvektoren[i][1][j] = tnvektoren;
			}
		}
		//in Datei ausgeben
		String text = "";
		//Spaltenzahl ermitteln
		double[] spaltenzahl = new double[lvvektoren.length];
		for (int j=0; j<lvvektoren.length; j++) //alle KZ
		{
			spaltenzahl[j] = 0;
			for (int k=0; k<lvvektoren[j][0].length; k++) //alle KM
			{
				if (teilsysteme[j].getKaeltemaschinen()[k].getRueckkuehler() instanceof Luftrueckkuehler)
					spaltenzahl[j] = spaltenzahl[j] + 8;
				else
					spaltenzahl[j] = spaltenzahl[j] + 12;
			}
		}
		//Teilsystemnamen in Kopfzeile 1
		
		for (int j=0; j<lvvektoren.length; j++) //alle KZ
		{
			text = text + teilsysteme[j].getName();
			for (int k=0; k<spaltenzahl[j]; k++)
			{
				text = text + "\t";
			}
		}
		text = text + "\n";
		//Kltemaschinen-, Rckkhler- und Subnetznamen in Kopfzeile 2
		for (int j=0; j<lvvektoren.length; j++) //alle KZ
		{
			for (int k=0; k<lvvektoren[j][0].length; k++) //alle KM
			{
				text = text + teilsysteme[j].getKaeltemaschinen()[k].getName();
				for (int l=0; l<5; l++)
				{
					text = text + "\t";
				}
				if (!(teilsysteme[j].getKaeltemaschinen()[k].getRueckkuehler() instanceof Luftrueckkuehler))
				{
					text = text + teilsysteme[j].getKaeltemaschinen()[k].getRueckkuehler().getName();
					for (int l=0; l<4; l++)
					{
						text = text + "\t";
					}
				}
			}
			for (int k=0; k<lvvektoren[j][1].length; k++) //alle TN
			{
				text = text + teilsysteme[j].getSubnetze()[k].getName();
				for (int l=0; l<3; l++)
				{
					text = text + "\t";
				}
			}
		}
		text = text + "\n";
		FileWriter fw = null;
		try {
			fw = new FileWriter(ziel);
			fw.write(text);
		} catch (IOException e1) {e1.printStackTrace();}
		StringBuffer buf = new StringBuffer(text);
		for (int j=0; j<lvvektoren.length; j++) //alle KZ
		{
			for (int k=0; k<lvvektoren[j][0].length; k++) //alle KM
			{
				buf.append("Klteleistung [kW]\t");
				buf.append("PLV [1]\t");
				buf.append("Stromverbrauch [kW]\t");
				buf.append("Wrmeverbrauch [kW]\t");
				buf.append("Rckkhlleistung [kW]\t");
				if (!(teilsysteme[j].getKaeltemaschinen()[k].getRueckkuehler() instanceof Luftrueckkuehler))
				{
					buf.append("Wasserverbrauch [m/h]\t");
					buf.append("Stromverbrauch [kW]\t");
					buf.append("f_R [1]\t");
					buf.append("t_W_E [C]\t");
				}
				
			}
			for (int k=0; k<lvvektoren[j][1].length; k++) //alle TN
			{
				buf.append("Volumenstrom [m/s]\t");
				buf.append("Druckverlust [Pa]\t");
				buf.append("Stromverbrauch [kW]\t");
			}
		}
		buf.append("\n");
		try {
			fw.write(buf.toString());
		} catch (IOException e) {e.printStackTrace();}
		buf.delete(0, buf.lastIndexOf(""));
		//Inhalte in die weiteren Zeilen
		DecimalFormat df = (DecimalFormat)DecimalFormat.getInstance(Locale.GERMAN);
		df.applyPattern( "#,###,##0.00" );
		for (int i=0; i<8760; i++)
		{
			for (int j=0; j<lvvektoren.length; j++) //alle KZ
			{
				for (int k=0; k<lvvektoren[j][0].length; k++) //alle KM
				{
					buf.append(df.format( new Double(lvvektoren[j][0][k][0][i])));
					buf.append("\t");
					buf.append(df.format( new Double(lvvektoren[j][0][k][1][i])));
					buf.append("\t");
					buf.append(df.format( new Double(lvvektoren[j][0][k][2][i])));
					buf.append("\t");
					buf.append(df.format( new Double(lvvektoren[j][0][k][3][i])));
					buf.append("\t");
					buf.append(df.format( new Double(lvvektoren[j][0][k][4][i])));
					buf.append("\t");
					if (!(teilsysteme[j].getKaeltemaschinen()[k].getRueckkuehler() instanceof Luftrueckkuehler))
					{
						buf.append(df.format( new Double(lvvektoren[j][0][k][5][i])));
						buf.append("\t");
						buf.append(df.format( new Double(lvvektoren[j][0][k][6][i])));
						buf.append("\t");
						buf.append(df.format( new Double(lvvektoren[j][0][k][7][i])));
						buf.append("\t");
						buf.append(df.format( new Double(lvvektoren[j][0][k][8][i])));
						buf.append("\t");
					}
					
				}
				for (int k=0; k<lvvektoren[j][1].length; k++) //alle TN
				{
					buf.append(df.format( new Double(lvvektoren[j][1][k][0][i])));
					buf.append("\t");
					buf.append(df.format( new Double(lvvektoren[j][1][k][1][i])));
					buf.append("\t");
					buf.append(df.format( new Double(lvvektoren[j][1][k][2][i])));
					buf.append("\t");
				}
			}
			buf.append("\n");
			try {
				fw.write(buf.toString());
			} catch (IOException e) {e.printStackTrace();}
			buf.delete(0, buf.lastIndexOf(""));
		}
		try {
			fw.flush();
			fw.close();
		} catch (IOException e) {			e.printStackTrace();		}

	}
	
	/**
	 * Dateianlage funktioniert bis jetzt nur unter Windows
	 */
	public void verbraucher_out()
	{
		//Vektoren sammeln
		double[] t_A = projekt.getKDS().getAussentemperatur();
		Verbraucher[] verbraucher = projekt.getVerbraucher();
		double[][] vektoren = new double[verbraucher.length][8760];
		for (int i=0; i<verbraucher.length; i++)
		{
			vektoren[i] = verbraucher[i].Kaelteleistung(t_A);
		}
		//in Datei ausgeben
		String text = "";
		//Teilsystemnamen in Kopfzeile 1
		for (int j=0; j<verbraucher.length; j++) //alle KZ
		{
			text = text + verbraucher[j].getName() + "\t";
		}
		text = text + "\n";
		StringBuffer buf = new StringBuffer(text);
		//Inhalte in die weiteren Zeilen
		DecimalFormat df = (DecimalFormat)DecimalFormat.getInstance(Locale.GERMAN);
		df.applyPattern( "#,###,##0.00" );
		for (int i=0; i<8760; i++)
		{
			for (int j=0; j<verbraucher.length; j++) //alle KZ
			{
				buf.append(df.format( new Double(vektoren[j][i])));
				buf.append("\t");
			}
			buf.append("\n");
		}
        String temp = "";
        File ziel = new File("verbraucher.dat");
        try {
        	ziel.createNewFile();
        	BufferedReader br = new BufferedReader(new StringReader(buf.toString()));
        	BufferedWriter bw = new BufferedWriter(new FileWriter(ziel));
            temp = br.readLine();
            while (temp != null)
            {
                bw.write(temp);
                bw.newLine();
                temp = br.readLine();
            }
            bw.close();
        } catch (IOException e) {e.printStackTrace();}

	}

	public Teilsystem[] getTeilsysteme() {
		return teilsysteme;
	}
	
}
