package de.elisabetheckstaedt.dipl.common;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.FileInputStream;
import java.io.InputStreamReader;

import de.elisabetheckstaedt.dipl.real.Luft;
import de.elisabetheckstaedt.dipl.real.Wasser;

/**
 * fasst Aussentemperatur und Wasserdampfgehalt zusammen
 * @author Elisabeth Eckstdt
 *
 */
public class Klimadatensatz {

	private String Name;
	/**
	 * Vektor 8760x[C]
	 */
	private double[] Aussentemperatur = new double[8760];
	
	/**
	 *  Vektor 8760x[g/kg] 
	 */
	private double[] Wasserdampfgehalt = new double[8760];
	
	/**
	 * @param name
	 * @param aussentemperatur
	 * @param wasserdampfgehalt
	 */
	public Klimadatensatz(String name, double[] aussentemperatur, double[] wasserdampfgehalt) {
		super();
		Name = name;
		Aussentemperatur = aussentemperatur;
		Wasserdampfgehalt = wasserdampfgehalt;
		//Zeichnen
		try
		{
			Zeichnen.Aussentemperatur(this, "Kontrollausgabe: Auentemperatur " + Name);
	//		Zeichnen.Feuchtkugeltemperatur(this, "T_F " + Name);
	//		Zeichnen.Wasserdampfgehalt(this, "x " + Name);
		}
		catch (Exception e)
		{
		}
	}

	/**
	 * Konstruktor<p>
	 * setzt Aussentemperatur auf den bergebenen Wert
	 */
	public Klimadatensatz(double[] Aussentemperatur)
	{
		super();
		this.Aussentemperatur = Aussentemperatur;
	}
	
	/**
	 * Konstruktor<p>
	 * setzt Aussentemperatur
	 * liest TRY01_Jahr.dat Datei entsprechend der Regionsnummer ein, diese muss im Unterordner "TRY" 
	 * liegen und entsprechend dem Beispiel benannt sein
	 * @param Region Nummer entsprechend Beschreibung der Testreferenzjahre des DWD [1...15]
	 */
	public Klimadatensatz(String Region)
	{
		super();
		String Datei = "TRY/TRY" + Region + "_Jahr.dat" ;
		try
		{
		    FileInputStream fstream = new FileInputStream(Datei);
		    DataInputStream in = new DataInputStream(fstream);
	        BufferedReader br = new BufferedReader(new InputStreamReader(in));
		    String Zeile;
		    //Read File Line By Line
		    for (int i=0; i<8760; i++)
		    {
		    	Zeile = br.readLine();
		    	String T_A = Zeile.substring(40, 44);
		    	Aussentemperatur[i] = Float.valueOf( T_A ).floatValue();
	        }
		    //Close the input stream
		    in.close();
		}
		catch (Exception e)
		{//Catch exception if any
			System.err.println("Error: " + e.getMessage());
		}
	}
	
	/**
	 * vereinfachte Formel
	 * @return feuchtkugeltemperatur [8760]
	 */
	public double[] feuchtkugeltemperatur()
	{
		double[] fkt = new double[8760];
		for (int i=0; i<8760; i++)
		{
			double t_A = Aussentemperatur[i];
			double h = new Luft().Waermekapazitaet(t_A) * t_A + Wasserdampfgehalt[i] / 1000 * (2500 + new Wasser().Waermekapazitaet(t_A) * t_A); //2500 ist Verdunstungsenthalpie bei 0 Grads
			fkt[i] = 0.00002 * Math.pow(h,3) - 0.0045 * Math.pow(h,2) + 0.6545 * h - 5.8202; 
		}
		return fkt;
	}
	
	public String getName() {
		return Name;
	}

	public void setName(String name) {
		Name = name;
	}
	public double[] getAussentemperatur() {
		return Aussentemperatur;
	}

	public double[] getWasserdampfgehalt() {
		return Wasserdampfgehalt;
	}
}