package de.elisabetheckstaedt.dipl.common;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * Diese Klasse dient der Summierung der verschiedenen Kostengruppen nach VDI
 * @author Elisabeth Eckstdt
 */
public abstract class Kostenstelle {
	
	protected final String Name;
	
	/**
	 * enthlt zustzlich Bauteilart
	 */
	protected final String XML_Name;
	
	/**
	 * Investition []
	 */
	protected double Investition;
	
	/**
	 * Zwischenspeicher Jahressumme []
	 */
	protected double Stromkosten;
	
	/**
	 * Zwischenspeicher Jahressumme []
	 */
	protected double Waermekosten;
	
	/**
	 * Zwischenspeicher Jahressumme []
	 */
	protected double Wasserkosten;
	
	/**
	 * Zwischenspeicher Jahressumme []
	 */
	private double Verbrauchsgebundene_Kosten;
	
	/**
	 * Zwischenspeicher Jahressumme []
	 */
	protected double Bedienungskosten;
	
	/**
	 * Zwischenspeicher Jahressumme []
	 */
	protected double Wartungskosten;
	
	/**
	 * Zwischenspeicher Jahressumme []
	 */
	protected double Inspektionskosten;
	
	/**
	 * Zwischenspeicher Jahressumme []
	 */
	private double Betriebsgebundene_Kosten;

	/**
	 * Investitionen, Ersatzinvestitionen, weniger Restwert incl. Zinsen []
	 */
	protected double Investitionskosten;
	
	/**
	 * Zwischenspeicher Jahressumme []
	 */
	protected double Instandsetzungskosten;
	
	/**
	 * Zwischenspeicher Jahressumme []
	 */
	private double Kapitalkosten;
	
	/**
	 * Zwischenspeicher Jahressumme []
	 */
	private double Kosten;
	
	/**
	 * Konstruktor<p>
	 * schreibt alle Instanzvariablen (werden auf 0 gesetzt)<p>
	 */
	protected Kostenstelle(String name) {
		super();
		Name = name;
		XML_Name = getBauteilart() + " " + name;
		Stromkosten = 0;
		Waermekosten = 0;
		Wasserkosten = 0;
		Verbrauchsgebundene_Kosten = 0;
		Bedienungskosten = 0;
		Wartungskosten = 0;
		Inspektionskosten = 0;
		Betriebsgebundene_Kosten = 0;
		Investitionskosten = 0;
		Instandsetzungskosten = 0;
		Kapitalkosten = 0;
		Kosten = 0;
	}
	/**
	 * schreibt {@link Kostenstelle#Kosten}<p>
	 * ruft {@link Kostenstelle#Kapitalkosten(double, double)}, 
	 * {@link Kostenstelle#Betriebsgebundene_Kosten(double, double, double)},
	 *  {@link Kostenstelle#Verbrauchsgebundene_Kosten(double[], double[], double[], double, double)}
	 * @param preisdaten <p>
	 * =  Kapitalkosten + Betriebsgebundene_Kosten + Verbrauchsgebundene_Kosten
	 */
	public void Kosten(Preisdaten preisdaten)
	{
		double Betrachtungszeitraum = preisdaten.getBetrachtungszeitraum();
		double Zinssatz = preisdaten.getZinssatz();
		double Stundenlohn = preisdaten.getStundenlohn();
		double[] Strompreis = preisdaten.getStrompreis();
		double[] Waermepreis = preisdaten.getWaermepreis();
		double[]Wasserpreis = preisdaten.getWasserpreis();
		double paf_lohn = preisdaten.getPaf_lohn();
		double paf_invest = preisdaten.getPaf_invest();
		double paf_energie = preisdaten.getPaf_energie();
		Kapitalkosten(Betrachtungszeitraum, Zinssatz, paf_invest);
		Betriebsgebundene_Kosten(Betrachtungszeitraum, Zinssatz, Stundenlohn, paf_invest, paf_lohn);
		Verbrauchsgebundene_Kosten(Strompreis, Waermepreis, Wasserpreis, Betrachtungszeitraum, Zinssatz, paf_energie);
		Kosten =  Kapitalkosten + Betriebsgebundene_Kosten + Verbrauchsgebundene_Kosten;
	}
	/**
	 * schreibt {@link Kostenstelle#Kapitalkosten}<p>
	 * ruft {@link Kostenstelle#Investitionskosten(double, double)}, {@link Kostenstelle#Instandsetzungskosten(double, double)}
	 * @param Betrachtungszeitraum [a]
	 * @param Zinssatz [1.05]
	 * @param paf_invest <p>
	 * = Investitionskosten + Instandsetzungskosten
	 */
	private void Kapitalkosten(double Betrachtungszeitraum, double Zinssatz, double paf_invest)
	{
		Investitionskosten(Betrachtungszeitraum, Zinssatz, paf_invest);
		Instandsetzungskosten(Betrachtungszeitraum, Zinssatz, paf_invest);
		Kapitalkosten = Investitionskosten + Instandsetzungskosten;
	}
	/**
	 * abstrakt
	 * schreibt {@link Kostenstelle#Investitionskosten}<p>
	 * @param Betrachtungszeitraum [a]
	 * @param Zinssatz [1.05]
	 * @param Preisnderungsfaktor fr Investititionen [1]
	 */
	protected abstract void Investitionskosten(double Betrachtungszeitraum, double Zinssatz, double paf_invest);
	/**
	 * abstrakt
	 * schreibt {@link Kostenstelle#Instandsetzungskosten}<p>
	 * @param Betrachtungszeitraum [a]
	 * @param Zinssatz [1.05]
	 * @return Instandsetzungskosten [/a]
	 */
	protected abstract void Instandsetzungskosten(double Betrachtungszeitraum, double Zinssatz, double paf_invest);
	

	/**
	 * schreibt {@link Kostenstelle#Betriebsgebundene_Kosten}<p>
	 * ruft {@link Kostenstelle#Bedienungskosten(double, double, double)}, 
	 * {@link Kostenstelle#Wartungskosten(double, double)}, 
	 * {@link Kostenstelle#Inspektionskosten(double, double)}
	 * @param Betrachtungszeitraum [a]
	 * @param Zinssatz [1.05]
	 * @param Stundenlohn [/h]
	 * @param paf_invest
	 * @param paf_lohn <p>
	 * = Bedienungskosten + Wartungskosten + Inspektionskosten
	 */
	private void Betriebsgebundene_Kosten(double Betrachtungszeitraum, double Zinssatz, double Stundenlohn 
			, double paf_invest, double paf_lohn) {
		Bedienungskosten(Betrachtungszeitraum, Zinssatz, Stundenlohn, paf_lohn);
		Wartungskosten(Betrachtungszeitraum, Zinssatz, paf_invest);
		Inspektionskosten(Betrachtungszeitraum, Zinssatz, Stundenlohn, paf_lohn);
		Betriebsgebundene_Kosten = Bedienungskosten + Wartungskosten + Inspektionskosten;
	}

	/**
	 * abstrakt
	 * schreibt {@link Kostenstelle#Bedienungskosten}<p>
	 * @param T Betrachtungszeitraum [a]
	 * @param q Zinssatz [1.05]
	 * @param Stundenlohn
	 * @param Preisnderungsfaktor Lohn [1.033]
	 */
	protected abstract void Bedienungskosten(double T, double q, double Stundenlohn, double paf_lohn);
	
	/**
	 * abstrakt
	 * schreibt {@link Kostenstelle#Wartungskosten}<p>
	 * @param T Betrachtungszeitraum [a]
	 * @param q Zinssatz [1.05]
	 * @param paf_invest
	 */
	protected abstract void Wartungskosten(double T, double q, double paf_invest);
	
	/**
	 * abstrakt
	 * schreibt {@link Kostenstelle#Inspektionskosten}<p>
	 * @param T Betrachtungszeitraum [a]
	 * @param q Zinssatz [1.05]
	 * @param Stundenlohn
	 * @param paf_lohn
	 */
	protected abstract void Inspektionskosten(double T, double q, double Stundenlohn, double paf_lohn);

	/**
	 * ruft {@link Kostenstelle#Stromkosten(double[], double, double, double)}, 
	 * {@link Kostenstelle#Waermekosten(double[], double, double, double)},
	 * {@link Kostenstelle#Wasserkosten(double[], double, double, double)}<p> 
	 * schreibt {@link Kostenstelle#Verbrauchsgebundene_Kosten}<p>
	 * @param Strompreis [/kWh]
	 * @param Waermepreis [/kWh]
	 * @param Wasserpreis [/m]
	 * @param Betrachtungszeitraum [a]
	 * @param Zinssatz [1.05]
	 * @param paf_energie <p>
	 * Stromkosten + Wasserkosten + Waermekosten
	 */
	private void Verbrauchsgebundene_Kosten(double[] Strompreis, double[] Waermepreis, double[]Wasserpreis, 
			double Betrachtungszeitraum, double Zinssatz, double paf_energie)
	{
		double Preisaenderungsfaktor = paf_energie;
		Stromkosten(Strompreis, Betrachtungszeitraum, Zinssatz, Preisaenderungsfaktor);
		Waermekosten(Waermepreis, Betrachtungszeitraum, Zinssatz, Preisaenderungsfaktor);
		Wasserkosten(Wasserpreis, Betrachtungszeitraum, Zinssatz, Preisaenderungsfaktor);
		Verbrauchsgebundene_Kosten = Stromkosten + Wasserkosten + Waermekosten;
	}
	protected abstract void Stromkosten(double[] Strompreis, double T, double q, double r);
	protected abstract void Waermekosten(double[] Waermereis, double T, double q, double r);
	protected abstract void Wasserkosten(double[] Wasserpreis, double T, double q, double r);

	/**
	 * Ergnzt den bergebenen XML-Baum um das das aktuelle Objekt
	 * Kostenbestandteile werden als Attribute eingefgt
	 * @param baum XML Dokument
	 * @param preisdaten
	 * @return XML-Knoten dieses Kostentrgers und ggf. seiner Kindobjekte
	 */
	protected Node KostenXMLout(Document baum, Preisdaten preisdaten)
	{
//		Kosten(preisdaten); //vorher mssen alle Kosten berechnet worden sein, hier findet nur auslesen statt
		Element aktElem = baum.createElement(getBauteilart());
		Helper.Wertattribut(aktElem, "Investition", getInvestitionskosten());
    	Helper.Wertattribut(aktElem, "Instandsetzung", getInstandsetzungskosten());
    	Helper.Wertattribut(aktElem, "Bedienung", getBedienungskosten());
    	Helper.Wertattribut(aktElem, "Wartung", getWartungskosten());
    	Helper.Wertattribut(aktElem, "Inspektion", getInspektionskosten());
    	Helper.Wertattribut(aktElem, "Stromkosten", getStromkosten());
        Helper.Wertattribut(aktElem, "Waermekosten", getWaermekosten());
        Helper.Wertattribut(aktElem, "Wasserkosten", getWasserkosten());
		aktElem.setAttribute("Erstinvestition", Investitionstext());
        aktElem.setAttribute("Paramter", Parametertext());
		return aktElem;
	}
	
	/**
	 * optimierte XML-Ausgabe fr das Einlesen als Excel-Pivot-Chart;. Mutterobjekte werden als Attribute hinzugefgt 
	 * damit mglichst flache Hierarchie in der XML_Datei
	 * @param baum XML_Dokument
	 * @param Wurzelknoten evtl. mit bereits vorhandenen Kindknoten
	 * @param teilsystemname Mutterobjekt
	 * @param subnetzname Mutterobjekt ggf.
	 * @param preisdaten
	 * @return Wurzelknoten diesem Kostentraeger als angefgter Kindknoten
	 */
	protected Node KostenXMLoutflach(Document baum, Node Wurzelknoten, String teilsystemname, String subnetzname, Preisdaten preisdaten)
	{
		Element aktElem = baum.createElement("Kostenstelle");
		aktElem.setAttribute("Typ", getBauteilart());
		aktElem.setAttribute("Name", getName());
		aktElem.setAttribute("Teilsystemnname", teilsystemname);
		aktElem.setAttribute("Subnetzname", subnetzname);
		aktElem.setAttribute("Kostenart1", "Kapitalgebundene_Kosten");
		aktElem.setAttribute("Kostenart2", "Investition");
		Helper.Wertattribut(aktElem, "Kosten", getInvestitionskosten());
		Wurzelknoten.appendChild(aktElem);

		aktElem = baum.createElement("Kostenstelle");
		aktElem.setAttribute("Typ", getBauteilart());
		aktElem.setAttribute("Name", getName());
		aktElem.setAttribute("Teilsystemnname", teilsystemname);
		aktElem.setAttribute("Subnetzname", subnetzname);
		aktElem.setAttribute("Kostenart1", "Kapitalgebundene_Kosten");
		aktElem.setAttribute("Kostenart2", "Instandsetzung");
		Helper.Wertattribut(aktElem, "Kosten", getInstandsetzungskosten());
		Wurzelknoten.appendChild(aktElem);
		
		aktElem = baum.createElement("Kostenstelle");
		aktElem.setAttribute("Typ", getBauteilart());
		aktElem.setAttribute("Name", getName());
		aktElem.setAttribute("Teilsystemnname", teilsystemname);
		aktElem.setAttribute("Subnetzname", subnetzname);
		aktElem.setAttribute("Kostenart1", "Betriebsgebundene_Kosten");
		aktElem.setAttribute("Kostenart2", "Bedienung");
		Helper.Wertattribut(aktElem, "Kosten", getBedienungskosten());
		Wurzelknoten.appendChild(aktElem);
		
		aktElem = baum.createElement("Kostenstelle");
		aktElem.setAttribute("Typ", getBauteilart());
		aktElem.setAttribute("Name", getName());
		aktElem.setAttribute("Teilsystemnname", teilsystemname);
		aktElem.setAttribute("Subnetzname", subnetzname);
		aktElem.setAttribute("Kostenart1", "Betriebsgebundene_Kosten");
		aktElem.setAttribute("Kostenart2", "Wartung");
		Helper.Wertattribut(aktElem, "Kosten", getWartungskosten());
		Wurzelknoten.appendChild(aktElem);
		
		aktElem = baum.createElement("Kostenstelle");
		aktElem.setAttribute("Typ", getBauteilart());
		aktElem.setAttribute("Name", getName());
		aktElem.setAttribute("Teilsystemnname", teilsystemname);
		aktElem.setAttribute("Subnetzname", subnetzname);
		aktElem.setAttribute("Kostenart1", "Betriebsgebundene_Kosten");
		aktElem.setAttribute("Kostenart2", "Inspektion");
		Helper.Wertattribut(aktElem, "Kosten", getInspektionskosten());
		Wurzelknoten.appendChild(aktElem);
		
		aktElem = baum.createElement("Kostenstelle");
		aktElem.setAttribute("Typ", getBauteilart());
		aktElem.setAttribute("Name", getName());
		aktElem.setAttribute("Teilsystemnname", teilsystemname);
		aktElem.setAttribute("Subnetzname", subnetzname);
		aktElem.setAttribute("Kostenart1", "Verbrauchsgebundene_Kosten");
		aktElem.setAttribute("Kostenart2", "Stromkosten");
		Helper.Wertattribut(aktElem, "Kosten", getStromkosten());
		Wurzelknoten.appendChild(aktElem);
		
		aktElem = baum.createElement("Kostenstelle");
		aktElem.setAttribute("Typ", getBauteilart());
		aktElem.setAttribute("Name", getName());
		aktElem.setAttribute("Teilsystemnname", teilsystemname);
		aktElem.setAttribute("Subnetzname", subnetzname);
		aktElem.setAttribute("Kostenart1", "Verbrauchsgebundene_Kosten");
		aktElem.setAttribute("Kostenart2", "Waermekosten");
		Helper.Wertattribut(aktElem, "Kosten", getWaermekosten());
		Wurzelknoten.appendChild(aktElem);
		
		aktElem = baum.createElement("Kostenstelle");
		aktElem.setAttribute("Typ", getBauteilart());
		aktElem.setAttribute("Name", getName());
		aktElem.setAttribute("Teilsystemnname", teilsystemname);
		aktElem.setAttribute("Subnetzname", subnetzname);
		aktElem.setAttribute("Kostenart1", "Verbrauchsgebundene_Kosten");
		aktElem.setAttribute("Kostenart2", "Wasserkosten");
		Helper.Wertattribut(aktElem, "Kosten", getWasserkosten());
		Wurzelknoten.appendChild(aktElem);
        
		return Wurzelknoten;
	}
	/**
	 * @return String mit wichtigsten Paramtern fr die Ausgabe in tief.xml
	 */
	public String Parametertext()
	{
		return "";
	}
	/**
	 * erscheint im XMLOut als nhere Info zum Objekt
	 * @return Erstinvestition als String euroformatiert
	 */
	public String Investitionstext()
	{
		return Helper.Euroformat(getInvestition(), 0);
	}
	public String toString()
	{
		return getName();
	}
	public double getInvestition() {
		return Investition;
	}
	public String getXML_Name() {
		return XML_Name;
	}
	public String getName() {
		return Name;
	}
	/**
	 * @return String Bauteilart
	 * fr XML-Name und Ausgabe in XML
	 */
	public abstract String getBauteilart();
	
	public double getStromkosten() {
		return Stromkosten;
	}
	public double getWaermekosten() {
		return Waermekosten;
	}
	public double getWasserkosten() {
		return Wasserkosten;
	}
	public double getBedienungskosten() {
		return Bedienungskosten;
	}
	public double getWartungskosten() {
		return Wartungskosten;
	}
	public double getInspektionskosten() {
		return Inspektionskosten;
	}
	public double getInstandsetzungskosten() {
		return Instandsetzungskosten;
	}
	public double getInvestitionskosten() {
		return Investitionskosten;
	}
	public double getBetriebsgebundene_Kosten() {
		return Betriebsgebundene_Kosten;
	}
	public double getKapitalkosten() {
		return Kapitalkosten;
	}
	public double getKosten() {
		return Kosten;
	}
	public double getVerbrauchsgebundene_Kosten() {
		return Verbrauchsgebundene_Kosten;
	}
}
