package de.elisabetheckstaedt.dipl.common;

import net.tinyos.matlab.MatClient;
import de.elisabetheckstaedt.dipl.real.Verbraucher;

/**
 * enthlt Funktionen zum Zeichnen in Matlab
 * @author Elisabeth Eckstdt
 */
public class Zeichnen {

	/**
	 * zeichnet den summierten Lastgang aller Verbraucher
	 * @param verbraucher
	 * @param kds
	 * @throws Exception 
	 */
	public static void lastgang(Verbraucher[] verbraucher, Klimadatensatz kds) throws Exception
	{
		Versorgungsaufgabe va = new Versorgungsaufgabe(verbraucher, kds);
		double[] x = new double[8760];
		String[] punktnamen = new String[8760];
		for (int i=0; i<8760; i++)
		{
			x[i] = i;
			punktnamen[i] = "";
		}
		Object[] Kurven = new Object[1];
		Object[] Kurve = new Object[4];
		Kurve[0] = x;
		Kurve[1] = va.Lastgang();
		Kurve[2] = punktnamen;
		Kurve[3] = "geordneter Lastgang";
		Kurven[0] = Kurve;
		String Fenstername = "Lastgang (summiert)";
		Object[] arg = {Kurven, Fenstername};
		MatClient mc = new MatClient();
		mc.sendCommand("lastgang", arg);
	}

	/**
	 * zeichnet den geordneten summierten Lastgang aller Verbraucher
	 * @param verbraucher
	 * @param kds
	 * @param Fenstername
	 * @throws Exception 
	 */
	public static void geordneter_lastgang(Verbraucher[] verbraucher, Klimadatensatz kds, String Fenstername) throws Exception
	{
		Versorgungsaufgabe va = new Versorgungsaufgabe(verbraucher, kds);
		double[] x = new double[8760];
		String[] punktnamen = new String[8760];
		for (int i=0; i<8760; i++)
		{
			x[i] = i;
			punktnamen[i] = "";
		}
		Object[] Kurven = new Object[1];
		Object[] Kurve = new Object[4];
		Kurve[0] = x;
		Kurve[1] = va.Lastgang_geordnet();
		Kurve[2] = punktnamen;
		Kurve[3] = "summierter geordneter Lastgang";
		Kurven[0] = Kurve;
		Object[] arg = {Kurven, Fenstername};
		MatClient mc = new MatClient();
		mc.sendCommand("lastgang", arg);
	}
	
	/**
	 * zeichnet alle einzelnen Lastgnge in ein Diagramm
	 * @param verbraucher
	 * @param kds
	 * @throws Exception 
	 */
	public static void lastgaenge(Verbraucher[] verbraucher, Klimadatensatz kds) throws Exception
	{
		double[] x = new double[8760];
		String[] punktnamen = new String[8760];
		for (int i=0; i<8760; i++)
		{
			x[i] = i;
			punktnamen[i] = "";
		}
		Object[] Kurven = new Object[verbraucher.length];
		for (int i=0; i<verbraucher.length; i++)
		{
			Object[] Kurve = new Object[4];
			Kurve[0] = x;
			Kurve[1] = verbraucher[i].Kaelteleistung(kds.getAussentemperatur());
			Kurve[2] = punktnamen;
			Kurve[3] = verbraucher[i].getName();
			Kurven[i] = Kurve;
		}
		String Fenstername = "Lastgnge";
		Object[] arg = {Kurven, Fenstername};
		MatClient mc = new MatClient();
		mc.sendCommand("lastgang", arg);
	}
	
	/**
	 * zeichnet alle geordneten Lasthnge der Verbraucher in ein Diagramm
	 * @param verbraucher
	 * @param kds
	 * @throws Exception 
	 */
	public static void geordnete_lastgaenge(Verbraucher[] verbraucher, Klimadatensatz kds) throws Exception
	{
		double[] x = new double[8760];
		String[] punktnamen = new String[8760];
		for (int i=0; i<8760; i++)
		{
			x[i] = i;
			punktnamen[i] = "";
		}
		Object[] Kurven = new Object[verbraucher.length];
		for (int i=0; i<verbraucher.length; i++)
		{
			Object[] Kurve = new Object[4];
			Kurve[0] = x;
			Kurve[1] = verbraucher[i].geordnete_Kaelteleistung(kds.getAussentemperatur());
			Kurve[2] = punktnamen;
			Kurve[3] = verbraucher[i].getName();
			Kurven[i] = Kurve;
		}
		String Fenstername = "Kontrollausgabe: geordnete Lastgnge aller Verbraucher";
		Object[] arg = {Kurven, Fenstername};
		MatClient mc = new MatClient();
		mc.sendCommand("lastgang", arg);
	}
	
	/**
	 * zeichnet alle geordneten Lasthnge der Verbraucher in ein Diagramm + den summierten
	 * @param verbraucher
	 * @param kds
	 * @param fenstername
	 * @throws Exception 
	 */
	public static void geordnete_lastgaenge_alle(Verbraucher[] verbraucher, Klimadatensatz kds, String fenstername) throws Exception
	{
		Versorgungsaufgabe va = new Versorgungsaufgabe(verbraucher, kds);
		double[] x = new double[8760];
		String[] punktnamen = new String[8760];
		for (int i=0; i<8760; i++)
		{
			x[i] = i;
			punktnamen[i] = "";
		}
		Object[] Kurven = new Object[verbraucher.length+1];
		Object[] Kurve = new Object[4];
		Kurve[0] = x;
		Kurve[1] = va.Lastgang_geordnet();
		Kurve[2] = punktnamen;
		Kurve[3] = "summiert";
		Kurven[0] = Kurve;
		for (int i=0; i<verbraucher.length; i++)
		{
			Kurve = new Object[4];
			Kurve[0] = x;
			Kurve[1] = verbraucher[i].geordnete_Kaelteleistung(kds.getAussentemperatur());
			Kurve[2] = punktnamen;
			Kurve[3] = verbraucher[i].getName();
			Kurven[i+1] = Kurve;
		}
		Object[] arg = {Kurven, fenstername};
		MatClient mc = new MatClient();
		mc.sendCommand("lastgang", arg);
	}
	
	/**
	 * zeichnet alle geordneten Lasthnge der Verbraucher in ein Diagramm + den summierten
	 * @param verbraucher
	 * @param kds
	 * @param fenstername
	 * @throws Exception 
	 */
	public static void lastgaenge_alle(Verbraucher[] verbraucher, Klimadatensatz kds, String fenstername) throws Exception
	{
		Versorgungsaufgabe va = new Versorgungsaufgabe(verbraucher, kds);
		double[] x = new double[8760];
		String[] punktnamen = new String[8760];
		for (int i=0; i<8760; i++)
		{
			x[i] = i;
			punktnamen[i] = "";
		}
		Object[] Kurven = new Object[verbraucher.length+1];
		Object[] Kurve = new Object[4];
		Kurve[0] = x;
		Kurve[1] = va.Lastgang();
		Kurve[2] = punktnamen;
		Kurve[3] = "summiert";
		Kurven[0] = Kurve;
		for (int i=0; i<verbraucher.length; i++)
		{
			Kurve = new Object[4];
			Kurve[0] = x;
			Kurve[1] = verbraucher[i].Kaelteleistung(kds.getAussentemperatur());
			Kurve[2] = punktnamen;
			Kurve[3] = verbraucher[i].getName();
			Kurven[i+1] = Kurve;
		}
		Object[] arg = {Kurven, fenstername};
		MatClient mc = new MatClient();
		mc.sendCommand("lastgang", arg);
	}
	
	/**
	 * @param verbraucher
	 * @param fenstername
	 * @throws Exception 
	 */
	public static void alle_Verbraucher (Verbraucher[] verbraucher, String fenstername) throws Exception
	{
		double[] x = new double[verbraucher.length];
		double[] y = new double[verbraucher.length];
		String[] punktnamen = new String[verbraucher.length];
		for (int i=0; i<verbraucher.length; i++)
		{
			x[i] = verbraucher[i].getOrt(0);
			y[i] = verbraucher[i].getOrt(1);
			punktnamen[i] = verbraucher[i].getName();
		}
		Object[] Kurven = new Object[1];
		Object[] Kurve = new Object[4];
		Kurve[0] = x;
		Kurve[1] = y;
		Kurve[2] = punktnamen;
		Kurve[3] = "Verbraucher";
		Kurven[0] = Kurve;
		Object[] arg = {Kurven, fenstername};
		MatClient mc = new MatClient();
		mc.sendCommand("Koordinaten", arg);
	}

	public static void Aussentemperatur(Klimadatensatz kds, String fenstername) throws Exception
	{
		double[] x = new double[8760];
		String[] punktnamen = new String[8760];
		for (int i=0; i<8760; i++)
		{
			x[i] = i;
			punktnamen[i] = "";
		}
		Object[] Kurven = new Object[1];
		Object[] Kurve = new Object[4];
		Kurve[0] = x;
		Kurve[1] = kds.getAussentemperatur();
		Kurve[2] = punktnamen;
		Kurve[3] = "Aussentemperatur";
		Kurven[0] = Kurve;
		Object[] arg = {Kurven, fenstername};
		MatClient mc = new MatClient();
		mc.sendCommand("Aussentemperatur", arg);
	}

	public static void vektor(double[] vektor, String name) throws Exception
	{
		double[] x = new double[8760];
		String[] punktnamen = new String[8760];
		for (int i=0; i<vektor.length; i++)
		{
			x[i] = i;
			punktnamen[i] = "";
		}
		Object[] Kurven = new Object[1];
		Object[] Kurve = new Object[4];
		Kurve[0] = x;
		Kurve[1] = vektor;
		Kurve[2] = punktnamen;
		Kurve[3] = name;
		Kurven[0] = Kurve;
		Object[] arg = {Kurven, name};
		MatClient mc = new MatClient();
		mc.sendCommand("Aussentemperatur", arg);
	}
	
	public static void Wasserdampfgehalt(Klimadatensatz kds, String fenstername) throws Exception
	{
		double[] x = new double[8760];
		String[] punktnamen = new String[8760];
		for (int i=0; i<8760; i++)
		{
			x[i] = i;
			punktnamen[i] = "";
		}
		Object[] Kurven = new Object[1];
		Object[] Kurve = new Object[4];
		Kurve[0] = x;
		Kurve[1] = kds.getWasserdampfgehalt();
		Kurve[2] = punktnamen;
		Kurve[3] = "Wasserdampfgehalt";
		Kurven[0] = Kurve;
		Object[] arg = {Kurven, fenstername};
		MatClient mc = new MatClient();
		mc.sendCommand("Aussentemperatur", arg);
	}

	public static void Feuchtkugeltemperatur(Klimadatensatz kds, String fenstername) throws Exception
	{
		double[] x = new double[8760];
		String[] punktnamen = new String[8760];
		for (int i=0; i<8760; i++)
		{
			x[i] = i;
			punktnamen[i] = "";
		}
		Object[] Kurven = new Object[1];
		Object[] Kurve = new Object[4];
		Kurve[0] = x;
		Kurve[1] = kds.feuchtkugeltemperatur();
		Kurve[2] = punktnamen;
		Kurve[3] = "Feuchtkugeltemperatur";
		Kurven[0] = Kurve;
		Object[] arg = {Kurven, fenstername};
		MatClient mc = new MatClient();
		mc.sendCommand("Aussentemperatur", arg);
	}
}
