package de.elisabetheckstaedt.dipl.real;

import java.util.Arrays;

import de.elisabetheckstaedt.dipl.common.Helper;

/**
 * reprsentiert (eine oder mehrere) Pumpen
 * @author Elisabeth Eckstdt
 */
public class Pumpe extends Stromverbraucher{

	/**
	 * [1]
	 */
	private double eta_ges;
	/**
	 * [m]
	 */
	private double Foerderhoehe;
	/**
	 * [m/s]
	 */
	private double Nennvolumenstrom;

	/**
	 * [m/s] Zwischenspeicher fr errechneten Volumenstrom
	 */
	private double[] vp;
	
	public Pumpe(String Name) {
		super(Name);
	}
	
	/**
	 * findet maximale Frderhhe und Volumenstrom<p>
	 * ruft dazu TN.Volumenstrom(), Druckverlust()<p>
	 * liest TN, TN.Verbraucher<p>
	 * ruft init(Volumenstrom, Frderhhe)
	 * @param tn
	 * @param T_A
	 */
	public void init(Subnetz tn, double[] T_A)
	{
		//Vorlauftemp des Subnetzes herausfinden
		double vlt = 100;
		for (int i=0; i<tn.getVerbraucher().length; i++)
		{
			if (tn.getVerbraucher()[i].getVorlauftemperatur() < vlt)
			{
				vlt = tn.getVerbraucher()[i].getVorlauftemperatur();
			}
		}
		//maximalen Volumenstrom finden
		vp = tn.volumenstrom(T_A, vlt);
		Arrays.sort(vp);
		Nennvolumenstrom = vp[8759];
		
		//maximalen Druckverlust finden
		double[] Druckverlust = tn.Druckverlust(vp); //TODO eigentlich Dichte mit aktuellen Temps aufrufen (siehe auch eins weiter unten
		Arrays.sort(Druckverlust);
		Foerderhoehe = Druckverlust[8759] / tn.getKaeltetraeger().Dichte(10) / 9.81 / 1000;
		init(Nennvolumenstrom, Foerderhoehe);
	}
	
	/**
	 * setzt Faktoren, eta_ges, NVp, NFH<p>
	 * ruft Investition()
	 * @param Nennvolumenstrom
	 * @param Foerderhoehe
	 */
	public void init(double Nennvolumenstrom, double Foerderhoehe)
	{
		Nutzungsdauer = 10; //a [VDI 2067:2007.03 Tab. A3 2.2.2]
		Instandsetzung = 1; //% der Investitionssumme pro a
		Wartung = 0; //% der Investitionssumme pro a
		Bedienung = 0; //h/a
		Inspektion = 0;
		eta_ges = 0.8;
		this.Nennvolumenstrom = Nennvolumenstrom;
		this.Foerderhoehe = Foerderhoehe;
		Investition();
	}
	
	/**
	 * angesetzt 300/kW Motorleistung
	 */
	private void Investition()
	{
		Investition = Foerderhoehe * 9.81 * 1000 * Nennvolumenstrom / eta_ges / 1000 * 300;
	}
	
	
	/**
	 * liest eta_ges
	 * @param Volumenstrom in m/s
	 * @param Druckverlust Pa
	 * @return Vektor des Stromverbrauchs in jeder Stunde des Jahres in kW
	 */
	public void Stromverbrauch(double[] Volumenstrom, double[] Druckverlust)
	{
		for (int i=0; i<8760; i++)
		{
			Stromverbrauch[i] = Volumenstrom[i] * Druckverlust[i] / eta_ges / 1000 ;
		}
	}
	/**
	 * ruft Investition()
	 * @param eta_ges
	 */
	public void setEta_ges(double eta_ges) {
		this.eta_ges = eta_ges;
		Investition();
	}
	/**
	 * ruft Investition()
	 * @param foerderhoehe
	 */
	public void setFoerderhoehe(double foerderhoehe) {
		Foerderhoehe = foerderhoehe;
		Investition();
	}
	/**
	 * ruft Investition()
	 * setzt Frderhhe auf Maximum des rufenden Subnetzes
	 * @param tn
	 */
	public void setFoerderhoehe(Subnetz tn) {
		double[] Druckverlust = tn.Druckverlust(vp);
		Arrays.sort(Druckverlust);
		Foerderhoehe = Druckverlust[8759] / tn.getKaeltetraeger().Dichte(10) / 9.81 / 1000;
		Investition();
	}

	/**
	 * ruft Investition()
	 * @param volumenstrom
	 */
	public void setVolumenstrom(double volumenstrom) {
		Nennvolumenstrom = volumenstrom;
		Investition();
	}

	public String Investitionstext()
	{
		return super.Investitionstext() + " Nennvolumenstrom: " + Helper.Zahlformat(Nennvolumenstrom,3) + " m/s" + 
		" Frderhhe: " + Helper.Zahlformat(Foerderhoehe,3) + " m";
	}

	public String Parametertext()
	{
		return "eta_ges " + eta_ges + " [1]";
	}

	public String getBauteilart()
	{
		return "Pumpe";
	}

	public double getFoerderhoehe() {
		return Foerderhoehe;
	}

	public double getVolumenstrom() {
		return Nennvolumenstrom;
	}

	public double getEta_ges()
	{
		return eta_ges;
	}
	
}
