package de.elisabetheckstaedt.dipl.real;

import de.elisabetheckstaedt.dipl.common.Klimadatensatz;

/**
 * Khlwasser wird durch Wrmetausch gekhlt, Verdunstungsklte wird nicht genutzt
 * @author Elisabeth Eckstdt
 */
public class Trockenrueckkuehler extends Wasserrueckkuehler {
	
	/**
	 * Konstruktor<p>
	 */
	public Trockenrueckkuehler(String name) {
		super(name);
		
	}

	public double[] f_R(Klimadatensatz kds, double[] t_W_E, double[] t_W_A) {
		double[] f_R = new double[8760];
		double Klammer; //Teillastluftmengenverhltnis
		double[] t_L = kds.getAussentemperatur();
		for (int i = 0; i<8760; i++)
		{
			Klammer = (t_W_A[i] - t_W_E[i]) / (t_W_A_0 - t_W_E_0) * (t_W_E_0 - 32) / (t_W_E[i] - t_L[i]);
			f_R[i] = Math.pow(Klammer, 2);
		}
		return f_R;
	}

	/**
	 * setzt Hilfsantriebe entsprechend DIN 18599-7 S. 59 auf 0.045<p>
	 * ruft WRK.init()
	 * @param km
	 */
	public void init(Kaeltemaschine km)
	{
		super.init(km);
		Hilfsantriebe = 0.045;
	}

	/**
	 * setzt Investition entsprechend IUTA HD
	 */
	protected void Investition() {
		Investition = 37.419 * Nennleistung + 312.93;
	}
	
	public String getBauteilart()
	{
		return "Trockenrueckkuehler";
	}
	


}
