package de.elisabetheckstaedt.dipl.real;

import java.util.Arrays;

import de.elisabetheckstaedt.dipl.common.Einzelobjekt;
import de.elisabetheckstaedt.dipl.common.Helper;


/**
 * reprsentiert Verbraucher (Anschlussstelle und Kltelast)
 * @author Elisabeth Eckstdt
 */
public abstract class Verbraucher extends Einzelobjekt{
	
	/**
	 * X und Y Koordinate in m
	 */
	protected final double[] Ort;
	
	/**
	 * bentigte Vorlauftemperatur [C] (Was ist maximal zulassig)
	 */
	protected final double Vorlauftemperatur;
	
	/**
	 * Druckverlust an der Hausanschlussstation [Pa]
	 */
	protected final double Druckverlust;
	
	protected double spitzenlast;

	/**
	 * Konstruktor<p>
	 * setzt Ort, Namen, Vorlauftemperatur
	 */
	protected Verbraucher(String name, double[] ort, double vorlauftemperatur) {
		super(name);
		Ort = ort;
		Vorlauftemperatur = vorlauftemperatur;
		Druckverlust = 20000; //TODO Konstante
	}
	
	/**
	 * setzt Faktoren und Spitzenlast<p>
	 * ruft Investition()
	 * @param Aussentemperatur
	 */
	public void init(double[] Aussentemperatur)
	{
		Nutzungsdauer = 20; //a [VDI 2067:2007.03 Tab. A3 2.1.2.2 (Plattenwrmetauscher)] //TODO HAusanschlussstation hat Faktoren vom Plattenwrmetauscher
		Instandsetzung = 2; //% der Investitionssumme pro a
		Wartung = 10; //% der Investitionssumme pro a
		Bedienung = 0; //h/a
		Inspektion = 0;
		spitzenlast = geordnete_Kaelteleistung(Aussentemperatur)[0];
		Investition();
	}
	
	/**
	 * setzt Kostenstelle.Investition je nach Spitzenlast
	 */
	protected void Investition()
	{
		if (spitzenlast > 500)
		{
			System.err.println(getXML_Name() + ": Spitzenlast grer als grtmgliche Hausanschlussstation: "
					+ spitzenlast + " kW");
			Investition = 40000;
		}
		else if (spitzenlast > 250)
		{
			Investition = 40000;
		}
		else if (spitzenlast > 50)
		{
			Investition = 25000;
		}
		else
		{
			Investition = 15000;
		}
	}

	public abstract double[] Kaelteleistung(double[] Aussentemperatur);
	
	public double[] geordnete_Kaelteleistung(double[] Aussentemperatur)
	{
		double[] kl = Kaelteleistung(Aussentemperatur);
		Arrays.sort(kl);
		return Helper.reverse(kl);
	}
	
	/**
	 * derzeit als konstant 12 implementiert
	 * @param 8760x[C] Aussentemperatur
	 * @return 8760x[C Ruecklauftemperatur
	 */
	public double[] Ruecklauftemperatur(double[] Aussentemperatur) //TODO Rcklauftemperatur von Auentemp
	{
		double[] Ruecklauftemperatur = new double[8760];
		Arrays.fill(Ruecklauftemperatur, 13);
		return Ruecklauftemperatur;
	}
	
	/**
	 * errechnet Massenstrom aus anliegender Kltelast, Temperaturdifferenz und Wrmekapazitt des Kltetrgers
	 * @param Kaeltetraeger: Objekt vom Typ Stoff
	 * @param Kaelteleistung 8760x[kW]
	 * @param Vorlauftemperatur [C] des Teilsystems, muss nicht unbedingt mit der des Verbrauchers bereinstimmen, da sie das Minimum aller Verbraucher ist
	 * @return Massenstrom 8760x[kg/s] variabel wenn die Rcklauftemperatur variabel ist
	 */
	public double[] Massenstrom(Stoff Kaeltetraeger, double[] Kaelteleistung, double Vorlauftemperatur, double[] Aussentemperatur)
	{
		double[] Massenstrom = new double[8760];
		double[] Qp = Kaelteleistung;
		double[] T_R = Ruecklauftemperatur(Aussentemperatur);
		double c0 = Kaeltetraeger.Waermekapazitaet(Vorlauftemperatur);
		for (int i=0; i<8760; i++)
		{
			double c = c0; //TODO eigentlich muss hier der Mittelwert von Vor- und RLT hin + Kaeltetraeger.Waermekapazitaet(T_R[i]))/2;
			Massenstrom[i] = Qp[i] / (T_R[i] - Vorlauftemperatur) / c;
		}
		return Massenstrom;
	}
	
	public abstract String toASCII();
	
	public String getBauteilart()
	{
		return "Verbraucher";
	}
	public String Investitionstext()
	{
		return super.Investitionstext() + "Spitzenlast: " + Helper.Zahlformat(spitzenlast,0) + " kW";
	}

	public void setOrt(double ortx, double orty) {
		Ort[0] = ortx;
		Ort[1] = orty;
	}
	
	public double getVorlauftemperatur() {
		return Vorlauftemperatur;
	}

	public double getOrt(int i) {
		return Ort[i];
	}

	/**
	 * @return Ort [X,Y] [m]
	 */
	public double[] getOrt() {
		return Ort;
	}

	public double getDruckverlust() {
		return Druckverlust;
	}
	
}