package de.elisabetheckstaedt.dipl.real;

import java.util.Arrays;

import de.elisabetheckstaedt.dipl.common.Klimadatensatz;

/**
 * reprsentiert alle Rckkhler, die Wasser an der MAschine vorbeischicken
 * @author Elisabeth Eckstdt
 */
public abstract class Wasserrueckkuehler extends Rueckkuehler{
	
	/**
	 * Eintrittstemperatur des Khlwassers in die Kltemaschine unter Auslegungsbedingungen [C]
	 */
	protected final double t_W_E_0;
	/**
	 * Austrittstemperatur des Khlwassers in die Kltemaschine unter Auslegungsbedingungen [C]
	 */
	protected final double t_W_A_0;

	/**
	 * setzt T_W_A/E_0 entsprechend der genauen Klasse des WRK
	 * @param name
	 */
	protected Wasserrueckkuehler(String name) {
		super(name);
		if (this instanceof Verdunstungsrueckkuehler)
		{
			this.t_W_E_0 = 27;
			this.t_W_A_0 = 33;
		}
		else
		{
			this.t_W_E_0 = 40;
			this.t_W_A_0 = 45;
		}
	}
	
	/**
	 * Formel entsprechnd din
	 * @param Kuehlleistung
	 * @return Eintrittstemperatur des Khlwassers in die Kltemaschine
	 */
	public double[] t_W_E (double[] Kuehlleistung)
	{
		double[] t_W_E = new double[8760];
		double[] t_W_A = t_W_A();
		for (int i=0; i<8760; i++)
		{
			t_W_E[i] = t_W_A[i] - Kuehlleistung[i] / Nennleistung * (t_W_A_0 - t_W_E_0);
		}
		return t_W_E;
	}
	
	/**
	 * @return 8760x Khlwasseraustrittstemperatur (derzeit konstant 33)
	 */
	public double[] t_W_A()
	{
		double[] t_W_A = new double[8760];
		Arrays.fill(t_W_A, 33);
		return t_W_A;
	}

	/**
	 * @param Klimadatensatz fr Feuchtkugeltemp
	 * @param t_W_E Khlwassereintritt in KM [C]
	 * @param t_W_A Khlwasseraustrott aus KM [C]
	 * @return Nutzungsfaktor Rckkhlung zum Zeitpunkt mit den bergebenen Temperaturen
	 */
	public abstract double[] f_R(Klimadatensatz kds, double[] t_W_E, double[] t_W_A);
	
	/**
	 * setzt Stromverbraucher.Stromverbrauch = Nennleistung * Hilfsantriebe * f_R
	 * @param Kuehlleistung
	 * @param kds
	 */
	public void Stromverbrauch(double[] Kuehlleistung, Klimadatensatz kds)
	{
		double[] t_W_E = t_W_E(Kuehlleistung);
		double[] f_R = f_R(kds, t_W_E ,t_W_A());
		for (int i=0; i<8760; i++)
		{
			Stromverbrauch[i] = Nennleistung * Hilfsantriebe * f_R[i] ; //ist 0 wenn Kuehlleistung = 0
		}
	}

}
